/* general */

function onButtonClicked(value) {
    document.sequence_form.submit_button.value=value;
    document.sequence_form.submit();
}

function insertAfter(parent, node, referenceNode) {
    parent.insertBefore(node, referenceNode.nextSibling);
}

function getElementsByClass(searchClass,node,tag) {
    var classElements = new Array();
    if ( node == null )
        node = document;
    if ( tag == null )
        tag = '*';
    var els = node.getElementsByTagName(tag);
    var elsLen = els.length;
    var pattern = new RegExp('(^|\\s)'+searchClass+'(\\s|$)');
    for (i = 0, j = 0; i < elsLen; i++) {
        if ( pattern.test(els[i].className) ) {
            classElements[j] = els[i];
            j++;
        }
    }
    return classElements;
}

function getTarget(event) {
    return event.target || event.srcElement; // IE uses srcElement instead of target
}

function resolveEvent(event) {
    return event || window.event; // IE requires us to look in window.event
}

function onLinkClicked(url) {
    document.sequence_form.click_url.value = url;
    document.sequence_form.submit_button.value = "click";
    document.sequence_form.submit();
    return false;
}

/* audio sequencing */

var masterSongList = null;

function initMasterSongList() {
    if (masterSongList == null) {
        masterSongList = getElementsByClass('song',document,null);
    }
    return masterSongList;
}

function whichSongIn(idToTest) {
    initMasterSongList();

    for (var songCount = 0; songCount < masterSongList.length; songCount++) {
        var masterSongIdentifier = masterSongList[songCount].getAttribute("id")
        if (masterSongIdentifier == idToTest) {
            return songCount;
        }
    }
    return -1;
}

function updateVisualMediaRelativeToPositionInSongs(songStartedIn, songFinishedIn) {
    if (songStartedIn == songFinishedIn) {
        return;
    }

    var firstSong = songStartedIn;
    var lastSong  = songFinishedIn;
    if (lastSong < firstSong) {
        firstSong = songFinishedIn;
        lastSong  = songStartedIn;
    }

    // process each song individually
    for (var songCount = 0; songCount < masterSongList.length; songCount++) {

        if (songCount < firstSong) {
            continue;
        }
        else if (songCount > lastSong) {
            break;
        }

        var currentSong = masterSongList[songCount];
        var currentSongDuration = 0.0;
        var currentSongConsumed = 0.0;
        currentSongDuration = Math.round(getSongDuration(currentSong) * 10000000) / 10000000;

        // since only the section with no song has this, there be any others
        if (currentSongDuration == 0.0) {
            break;
        }

        var visualMediaInSong = getElementsByClass('visualMediaItem', currentSong, null);

        var mediaToMove = [];
        var mediaToMoveCount = 0;

        for (var visualMediaCount = 0; visualMediaCount < visualMediaInSong.length; visualMediaCount++) {
            var visualMedia = visualMediaInSong[visualMediaCount];
            var mediaToAddDuration = parseFloat(visualMedia.getAttribute("duration"));
            currentSongConsumed = Math.round((currentSongConsumed + mediaToAddDuration) * 1000) / 1000;

            if (currentSongDuration > 0) {
                if ((currentSongConsumed - currentSongDuration) >= mediaToAddDuration) {
                    mediaToMove[mediaToMoveCount] = visualMedia;
                    mediaToMoveCount++;
                    var parentNode = visualMedia.parentNode;
                    parentNode.removeChild(visualMedia);
                }
            }
        }

        // this handles overflowing downward, if something was dragged from a lower song to a upper one
        if (mediaToMove.length > 0) {
            var nextSong = masterSongList[songCount + 1];

            for (var i = 0; i < mediaToMove.length; i++) {
                nextSong.insertBefore(mediaToMove[mediaToMove.length-1-i], nextSong.firstChild);
            }
        }

        // this handles overflowing upward, if something was dragged from a upper song to a lower one
        else if ((songCount < masterSongList.length-1) && currentSongConsumed < currentSongDuration) {
            var mediaToMove = [];
            var mediaToMoveCount = 0;

            var nextSong = masterSongList[songCount + 1];
            var visualMediaInNextSong = getElementsByClass('visualMediaItem', nextSong, null);

            for (var j = 0; j < visualMediaInNextSong.length; j++) {
                var mediaInNextSong = visualMediaInNextSong[j];
                var duration = parseFloat(mediaInNextSong.getAttribute("duration"));
                var expectedAmountOfCurrentSongConsumed = Math.round((currentSongConsumed + duration) * 1000) / 1000;

                if (expectedAmountOfCurrentSongConsumed <= currentSongDuration) {
                    currentSong.appendChild(mediaInNextSong);

                    currentSongConsumed = Math.round((currentSongConsumed + duration) * 1000) / 1000;
                }
                else {
                    break;
                }
            }
        }
    }
}

function getSongDuration(song) {
    return song.getAttribute("duration");
}

/* visual sequencing */

var prevValue;

function forceInRange(element, value) {
    if(value < 1) {
        return 1;
    }
    var maxValue = mediaCounts[element.getAttribute("media_type")];
    if(value > maxValue) {
        return maxValue;
    }
    return value;
}

function onSequenceBoxFocus(focusElement) {
    prevValue = focusElement.value;
}

function onSequenceBoxChange(changedElement) {
    if(isNaN(changedElement.value)) {
        changedElement.value = prevValue;
        return;
    }

    prevInt = parseInt(prevValue, 10);
    newInt = parseInt(changedElement.value, 10);
    onSequenceBoxChangeToValue(changedElement, prevInt, newInt);
}

function onSequenceBoxChangeToValue(changedElement, prevInt, newInt) {
    var mediaType = changedElement.getAttribute("media_type");

    newInt = forceInRange(changedElement, newInt);
    changedElement.value = newInt;

    var startRange, endRange, increment;
    if(newInt < prevInt) {
        startRange = newInt;
        endRange = prevInt - 1;
        increment = 1;
    } else {
        startRange = prevInt + 1;
        endRange = newInt;
        increment = -1;
    }

    var inputs = document.getElementById("sequence_section").getElementsByTagName("input");
    elementCount = inputs.length;
    for(index = 0; index < elementCount; index++) {
        element = inputs[index];
        if(element != changedElement && element.getAttribute("media_type") == mediaType) {
            var elementValue = parseInt(element.value);
            if(elementValue >= startRange && elementValue <= endRange) {
                element.value = elementValue + increment;
            }
        }
    }
}

function onSequenceBoxIncrement(elementName, increment) {
    element = document.forms["sequence_form"].elements[elementName];
    prevInt = parseInt(element.value, 10);
    newInt = prevInt + increment;
    onSequenceBoxChangeToValue(element, prevInt, newInt);
}

function onSequenceBoxKeyDown(element, event) {
    if(event.keyCode == 13) {
        onSequenceBoxChange(element);
        <!-- to force an update of prevValue to the current value -->
        onSequenceBoxFocus(element);
    }
}

var state="none";
var prevDragObject;
var dragObject;
var dragDropMirrorContainer;

function findObjectPosition(obj) {
	var currentLeft = currentTop = currentWidth = currentHeight = 0;

	if (obj.offsetParent) {
		currentLeft   = obj.offsetLeft;
		currentTop    = obj.offsetTop;
		currentWidth  = obj.offsetWidth;
		currentHeight = obj.offsetHeight;
		
		while (obj = obj.offsetParent) {
			currentLeft += obj.offsetLeft;
			currentTop  += obj.offsetTop;
		}
	}
	return [currentLeft,currentTop,currentWidth,currentHeight];
}

function getPrevDragObject(event) {
    if (state != "dragging") {
        return null;
    }

    event = resolveEvent(event);

	var mouseX = event.clientX;
	var mouseY = (event.clientY + document.body.scrollTop);		

	// adjust for the scrolling ONLY on IE
    if(document.all) {
		var scroller = document.getElementById("mainScroller");
		mouseY = mouseY + scroller.scrollTop;
    }

	var previousClosest = -1;
	var i;
	
	for (i=0; i < visualMediaItems.length; i++){	
	
		var positionValues = findObjectPosition(visualMediaItems[i]);
		var left   = positionValues[0];
		var top    = positionValues[1];
		var right  = positionValues[0] + positionValues[2]; 
		var bottom = positionValues[1] + positionValues[3]; 

		// don't continue since the mouse is above any legitimate items
		if (mouseY < top) {
			if (previousClosest > -1)
				return visualMediaItems[previousClosest];
			break;
		}
		// the mouse drag isn't pertinent to this row
		else if (mouseY > bottom) {
			continue;
		}
		// it is in the vertical range
		else if ((mouseY > top) && (mouseY < bottom)) {
			previousClosest = i;	
		}
	}

	// this handles the last one
	if ( (i == visualMediaItems.length) && (mouseY > top) && (mouseY < bottom) && (mouseX > right)  ) {
		return visualMediaItems[i-1];
	}
	
	return null;
}

function isBefore(dragObject, obj) {
    var changedElement = dragObject.getElementsByTagName("input")[0];
    var prevInt = parseInt(changedElement.value);
    var newInt = parseInt(obj.getElementsByTagName("input")[0].value);
    return prevInt < newInt;
}

function onMouseDownObject(obj, event) {
    var target = getTarget(event);
    var nodeName = target.nodeName.toUpperCase();
    if(nodeName == "INPUT") {
        return;
    }
    if(nodeName == "IMG" && target.className.indexOf("dragHandle") != 0) {
        return;
    }

    state = "dragging";
    dragObject = obj;

    // onmousedown, make item being moved semitransparent
    if (browser.isGecko || browser.isNS || browser.isMozilla || browser.isNScompatible) {
        var dragObjectDiv = dragObject.childNodes[1];
    } else {
        var dragObjectDiv = dragObject.children[0];
    }
    
    dragObjectDiv.style.opacity = 0.5;
    dragObjectDiv.style.MozOpacity = 0.5;
    dragObjectDiv.style.filter = "alpha(opacity=50)";

	startMirroring(event, dragObjectDiv);

    return false;
}

function onMouseMoveObject(obj, event) {
    if(state != "dragging") {
        return;
    }

    commonMouseMoveLogic(event);

    var indicatorClass = isBefore(dragObject, obj) ? "insertRight" : "insertLeft";
    
    obj.className="visualMediaItemOn " + indicatorClass;
    prevDragObject = obj;
    
    event.cancelBubble = true;
}

function onMouseMoveGlobal(event) {
    if(state != "dragging") {
        return;
    }

    event = resolveEvent(event);

    commonMouseMoveLogic(event);
    
    obj = getPrevDragObject(event);
	if (obj != null) {
	    var indicatorClass = "insertRight";
	    obj.className="visualMediaItemOn " + indicatorClass;
	    prevDragObject = obj;
	}
	else {
	}
}

function onMouseUpObject(obj, event, insertAfter) {

    if(state != "dragging") {
        return;
    }

    // onmouseup, restore opacity of item being moved
    if (browser.isGecko || browser.isNS || browser.isMozilla || browser.isNScompatible) {
        var dragObjectDiv = dragObject.childNodes[1];
    } else {
        var dragObjectDiv = dragObject.children[0];
    }
    
    dragObjectDiv.style.opacity = 1.0;
    dragObjectDiv.style.MozOpacity = 1.0;
    dragObjectDiv.style.filter = "alpha(opacity=100)";

	if (obj == null) {
		return;
	}

    resetPrevDragObject();

    songStartedIn = whichSongIn(dragObject.parentNode.getAttribute("id"));

    var changedElement = dragObject.getElementsByTagName("input")[0];
    var prevInt = parseInt(changedElement.value);
    
	var newInt;
   	newInt = parseInt(obj.getElementsByTagName("input")[0].value);

    var insertPosition = obj;

	if(insertAfter && prevInt > newInt) {
		insertPosition = insertPosition.nextSibling;
		newInt++;
	} else if(prevInt < newInt) {
        insertPosition = insertPosition.nextSibling;
    }
    
	obj.parentNode.insertBefore(dragObject, insertPosition);

    onSequenceBoxChangeToValue(changedElement, prevInt, newInt);

    songFinishedIn = whichSongIn(dragObject.parentNode.getAttribute("id"));

    updateVisualMediaRelativeToPositionInSongs(songStartedIn, songFinishedIn);

    endDrag();

	endMirroring();

	getVisualMediaItems();	
	
    event.cancelBubble = true;
}

function startMirroring(event,dragObjectDiv) {
	var dragDropMirrorDiv = document.createElement('div');
	dragDropMirrorDiv.setAttribute('id','dragDropMirror');	
	dragDropMirrorDiv.className = dragObjectDiv.className;
	dragDropMirrorDiv.innerHTML = dragObjectDiv.innerHTML;
    dragDropMirrorContainer.appendChild(dragDropMirrorDiv);
	
	dragDropMirrorContainer.style.display = 'block';

	setMirroringPosition(event);	
}

function setMirroringPosition(event) {
    event = resolveEvent(event);
	
	// the 10 is to avoid masking AND the 60 was for appearance
	dragDropMirrorContainer.style.left = (event.clientX + 10) + 'px';
	dragDropMirrorContainer.style.top  = (event.clientY - 60 + document.body.scrollTop)  + 'px';
}

function endMirroring() {
	dragDropMirrorContainer.innerHTML='';
	dragDropMirrorContainer.style.display='none';
}

function onMouseUpGlobal(event) {
    event = resolveEvent(event);

	endMirroring();

	onMouseUpObject(prevDragObject, event, true);

    endDrag();
    
	getVisualMediaItems();	
}

/** Allow selecting text within input box. */
function onSelectStartObject(event) {
    return getTarget(event).nodeName.toUpperCase() == "INPUT";
}

function commonMouseMoveLogic(event) {
    resetPrevDragObject();

    scrollWindow(event);
    
    setMirroringPosition(event);
}

function scrollWindow(event) {
    if(document.all) {
        // on IE, scroll viewport
        var scroller = document.getElementById("mainScroller");
        var yPos = event.clientY;
        var parent = scroller;
        while(parent != null) {
            yPos -= parent.offsetTop;
            parent = parent.offsetParent;
        }

        if(yPos < 30) {
            var scrollAmount = 30 - yPos;
            scrollAmount = Math.min(scrollAmount, 30);
            scroller.scrollTop = scroller.scrollTop - scrollAmount;
        } else if(yPos > scroller.clientHeight - 30) {
            var scrollAmount = yPos - (scroller.clientHeight - 30);
            scrollAmount = Math.min(scrollAmount, 30);
            scroller.scrollTop = scroller.scrollTop + scrollAmount;
        }
    } else {
        // on Firefox, scroll entire window
        if(event.clientY < 30) {
            var scrollAmount = 30 - event.clientY;
            scrollAmount = Math.min(scrollAmount, 30);
            window.scrollBy(0, -scrollAmount);
        } else if(event.clientY > document.body.clientHeight - 30) {
            var scrollAmount = event.clientY - (document.body.clientHeight - 30);
            scrollAmount = Math.min(scrollAmount, 30);
            window.scrollBy(0, scrollAmount);
        }
    }
}

function resetPrevDragObject() {
    if(prevDragObject == null) {
        return ;
    }

    prevDragObject.className="visualMediaItem";
    prevDragObject = null;
}

function endDrag() {
    state = "none";
}

function getVisualMediaItems() {
	visualMediaItems = getElementsByClass('visualMediaItem',document,'div');
}

function initSequence() {
    document.body.onmousemove=onMouseMoveGlobal;
    document.body.onmouseup=onMouseUpGlobal;
    
   	dragDropMirrorContainer = document.getElementById('dragDropMirrorContainer');
	dragDropMirrorContainer.className='dragDropMirrorContainer';	

	getVisualMediaItems();
}
